@@ -27,8 +27,9 @@ import traceback
 import signal
 from platform import node
 from optparse import OptionParser, OptionGroup
-from pynag.Utils import PerfData, PynagError, reconsile_threshold
+from pynag.Utils import PerfData, PynagError, reconsile_threshold, runCommand
 from pynag.Parsers import ExtraOptsParser
+import pynag.Utils
 import new_threshold_syntax
 
 # Map the return codes
@@ -261,46 +262,15 @@ class simple:
         """ deprecated. Use pynag.Plugins.check_range() """
         return check_range(value=value, range_threshold=range_threshold)
 
-    def send_nsca(self, code, message, ncsahost, hostname=node(), service=None):
-        """
-        Send data via send_nsca for passive service checks
-        """
-
-        # Execute send_nsca
-        from popen2 import Popen3
-        command = "send_nsca -H %s" % ncsahost
-        p = Popen3(command,  capturestderr=True)
-
-        # Service check
-        if service:
-            print >>p.tochild, "%s	%s	%s	%s %s" % (
-                hostname, service, code, message, self.perfdata_string())
-        # Host check, omit service_description
-        else:
-            print >>p.tochild, "%s	%s	%s %s" % (
-                hostname, code, message, self.perfdata_string())
-
-        # Send eof
-        # TODO, support multiple statuses ?
-        p.tochild.close()
-
-        # Save output incase we have an error
-        nsca_output = ''
-        for line in p.fromchild.readlines():
-            nsca_output += line
-
-        # Wait for send_nsca to exit
-        returncode = p.wait()
-        returncode = os.WEXITSTATUS(returncode)
-
-        # Problem with running nsca
-        if returncode != 0:
-            if returncode == 127:
-                raise Exception("Could not find send_nsca in path")
-            else:
-                raise Exception(
-                    "returncode: %i\n%s" % (returncode, nsca_output))
+    def send_nsca(self, *args, **kwargs):
+        """ Wrapper around pynag.Utils.send_nsca - here for backwards compatibility """
 
+        # Previous versions of this method had a typo where one argument was called
+        # ncsahost instead of nscahost. We will maintain backwards compatibility here.
+        if 'ncsahost' in kwargs and not 'nscahost' in kwargs:
+            kwargs['nscahost'] = kwargs['ncsahost']
+            del kwargs['ncsahost']
+        pynag.Utils.send_nsca(*args, **kwargs)
         return 0
 
     def nagios_exit(self, code_text, message):
@@ -312,7 +282,7 @@ class simple:
         code = self.code_string2int(code_text)
 
         # This should be one line (or more in nagios 3)
-        print "%s: %s %s" % (self.status_text[code], message, self.perfdata_string())
+        print("%s: %s %s" % (self.status_text[code], message, self.perfdata_string()))
         sys.exit(code)
 
     def perfdata_string(self):
@@ -1034,7 +1004,7 @@ class PluginHelper:
         plugin_output = self.get_plugin_output(
             exit_code=exit_code, summary=summary, long_output=long_output, perfdata=perfdata)
 
-        print plugin_output
+        print(plugin_output)
         sys.exit(exit_code)
 
     def check_metric(self, metric_name, thresholds):
@@ -1220,7 +1190,7 @@ class PluginHelper:
         >>> p = PluginHelper()
         >>> p.add_status(ok)
         >>> p.add_summary('Test')
-        >>> print p
+        >>> print(p)
         OK - Test
         """
         return self.get_plugin_output()
