@@ -33,10 +33,10 @@ from platform import node
 from getpass import getuser
 import datetime
 import pynag.Plugins
+import sys
 
 rlock = threading.RLock()
 
-
 class PynagError(Exception):
 
     """ The default pynag exception.
@@ -56,18 +56,29 @@ class PynagError(Exception):
             Exception.__init__(self, message, *args, **kwargs)
 
 
-def runCommand(command, raise_error_on_fail=False):
-    """ Run command from the shell prompt. Wrapper around subprocess.
+def runCommand(command, raise_error_on_fail=False, shell=True, env=None):
+    """ 
+    Run command from the shell prompt. Wrapper around subprocess.
 
-     Arguments:
-         command: string containing the command line to run
-         raise_error_on_fail: Raise PynagError if returncode >0
-     Returns:
-         stdout/stderr of the command run
-     Raises:
-         PynagError if returncode > 0
-     """
-    proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE,)
+    Arguments:
+        command (str): string containing the command line to run
+        raise_error_on_fail (bool): Raise PynagError if returncode > 0
+
+    Returns:
+        string: stdout/stderr of the command run
+
+    Raises:
+        PynagError if returncode > 0
+    """
+    run_env = environ.copy()
+    # Merge dict into environ
+    if env:
+        run_env.update(env)
+    proc = subprocess.Popen(command,
+                            shell=shell,
+                            stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE,
+                            env=run_env)
     stdout, stderr = proc.communicate('through stdin to stdout')
     result = proc.returncode, stdout, stderr
     if proc.returncode > 0 and raise_error_on_fail == True:
@@ -90,11 +101,14 @@ class GitRepo(object):
         Python Wrapper around Git command line.
 
         Arguments:
-          Directory   -- Which directory does the git repo reside in (i.e. '/etc/nagios')
-          auto_init   -- If True and directory does not contain a git repo, create it automatically
-          author_name -- Full name of the author making changes
-          author_email -- Email used for commit messages, if None, then use username@hostname
 
+            Directory (str): Path to the directory does the git repo reside in (i.e. '/etc/nagios')
+
+            auto_init (bool): If True and directory does not contain a git repo, create it automatically
+
+            author_name (str): Full name of the author making changes
+
+            author_email (str): Email used for commit messages, if None, then use username@hostname
         """
 
         self.directory = directory
@@ -119,7 +133,8 @@ class GitRepo(object):
         if auto_init:
             try:
                 self._run_command('git status --short')
-            except PynagError, e:
+            except PynagError:
+                t, e = sys.exc_info()[:2]
                 if e.errorcode == 128:
                     self.init()
             #self._run_command('git status --short')
@@ -127,15 +142,25 @@ class GitRepo(object):
         self._is_dirty = self.is_dirty  # Backwards compatibility
 
     def _update_author(self):
-        """ Updates environment variables GIT_AUTHOR_NAME and EMAIL
+        """ 
+        Updates environment variables GIT_AUTHOR_NAME and EMAIL
 
-        Returns: None
+        Returns:
+            None
         """
         environ['GIT_AUTHOR_NAME'] = self.author_name
         environ['GIT_AUTHOR_EMAIL'] = self.author_email.strip('<').strip('>')
 
     def _run_command(self, command):
-        """ Run a specified command from the command line. Return stdout """
+        """ 
+        Run a specified command from the command line. Return stdout 
+        
+        Arguments:
+            command (str): command to execute
+
+        Returns:
+            stdout of the executed command
+        """
         import subprocess
         import os
         cwd = self.directory
@@ -149,16 +174,34 @@ class GitRepo(object):
         return stdout
 
     def is_up_to_date(self):
-        """ Returns True if all files in git repo are fully commited """
+        """ 
+        Returns True if all files in git repo are fully commited 
+        
+        Returns:
+            bool. Git repo is up-to-date
+
+                True -- All files are commited
+
+                False -- At least one file is not commited
+        """
         return len(self.get_uncommited_files()) == 0
 
     def get_valid_commits(self):
-        """ Returns a list of all commit ids from git log
+        """ 
+        Returns a list of all commit ids from git log
+
+        Returns:
+            List of all valid commit hashes
         """
         return map(lambda x: x.get('hash'), self.log())
 
     def get_uncommited_files(self):
-        """ Returns a list of files that are have unstaged changes """
+        """ 
+        Returns a list of files that are have unstaged changes 
+        
+        Returns:
+            List. All files that have unstaged changes.
+        """
         output = self._run_command("git status --porcelain")
         result = []
         for line in output.split('\n'):
@@ -185,6 +228,12 @@ class GitRepo(object):
 
         Any arguments provided will be passed directly to pynag.Utils.grep() to filter the results.
 
+        Arguments:
+            kwargs: Arguments passed to pynag.Utils.grep()
+
+        Returns:
+            List of dicts. Log of previous commits.
+
         Examples:
           self.log(author_name='nagiosadmin')
           self.log(comment__contains='localhost')
@@ -204,9 +253,19 @@ class GitRepo(object):
         return grep(result, **kwargs)
 
     def diff(self, commit_id_or_filename=None):
-        """ Returns diff (as outputted by "git diff") for filename or commit id.
+        """ 
+        Returns diff (as outputted by "git diff") for filename or commit id.
 
         If commit_id_or_filename is not specified. show diff against all uncommited files.
+
+        Arguments:
+            commit_id_or_filename (str): git commit id or file to diff with
+
+        Returns:
+            str. git diff for filename or commit id
+
+        Raises:
+            PynagError: Invalid commit id or filename was given
         """
         if commit_id_or_filename in ('', None):
             command = "git diff"
@@ -222,7 +281,17 @@ class GitRepo(object):
         return self._run_command(command)
 
     def show(self, commit_id,):
-        """ Returns output from "git show" for a specified commit_id
+        """ 
+        Returns output from "git show" for a specified commit_id
+
+        Arguments:
+            commit_id (str): Commit id of the commit to display (``git show``)
+
+        Returns:
+            str. Output of ``git show commit_id``
+
+        Raises: 
+            PynagError: Invalid commit_id was given
         """
         if commit_id not in self.get_valid_commits():
             raise PynagError("%s is not a valid commit id" % commit_id)
@@ -251,7 +320,22 @@ class GitRepo(object):
         return self.commit(message=message, filelist=filelist)
 
     def pre_save(self, object_definition, message):
-        """ Commits object_definition.get_filename() if it has any changes """
+        """ 
+        Commits object_definition.get_filename() if it has any changes. This
+        function is called by :py:class:`pynag.Model.EventHandlers` before
+        calling :py:meth:`pynag.Utils.GitRepo.save`
+        
+        Arguments:
+
+            object_definition (pynag.Model.ObjectDefinition): object to commit changes
+
+            message (str): git commit message as specified in  ``git commit -m``
+
+        A message from the authors:
+            *"Since this is still here, either i forgot to remove it, or because
+            it is here for backwards compatibility, palli"*
+
+        """
         filename = object_definition.get_filename()
         if self.is_dirty(filename):
             self._git_add(filename)
@@ -260,6 +344,17 @@ class GitRepo(object):
                             (object_definition.object_type, object_definition.get_shortname()))
 
     def save(self, object_definition, message):
+        """
+        Commits object_definition.get_filename() if it has any changes. This
+        function is called by :py:class:`pynag.Model.EventHandlers`
+
+        Arguments:
+
+            object_definition (pynag.Model.ObjectDefinition): object to commit changes
+
+            message (str): git commit message as specified in  ``git commit -m``
+
+        """
         filename = object_definition.get_filename()
         if len(self.messages) > 0:
             message = [message, '\n'] + self.messages
@@ -270,13 +365,30 @@ class GitRepo(object):
         self.messages = []
 
     def is_dirty(self, filename):
-        """ Returns True if filename needs to be committed to git """
+        """
+        Returns True if filename needs to be committed to git 
+
+        Arguments: 
+
+            filename (str): file to check
+        """
         command = "git status --porcelain '%s'" % filename
         output = self._run_command(command)
         # Return True if there is any output
         return len(output) > 0
 
     def write(self, object_definition, message):
+        """
+        This method is called whenever :py:class:`pynag.Model.EventHandlers`
+        is called.
+
+        Arguments
+
+            object_definition (pynag.Model.ObjectDefinition): Object to write to file.
+
+            message (str): git commit message as specified in  ``git commit -m``
+
+        """
         # When write is called ( something was written to file )
         # We will log it in a buffer, and commit when save() is called.
         self.messages.append(" * %s" % message)
@@ -291,11 +403,18 @@ class GitRepo(object):
         """ Commit files with "git commit"
 
         Arguments:
-          message      -- Message used for the git commit
-          filelist     -- List of filenames to commit (if None, then commit all files in the repo)
-          author       -- Author to use for git commit. If any is specified, overwrite self.author_name and self.author_email
+
+            message (str): Message used for the git commit
+
+            filelist (list of strings): List of filenames to commit (if None, 
+                then commit all files in the repo)
+            author (str): Author to use for git commit. If any is specified, 
+                overwrite self.author_name and self.author_email
+
         Returns:
-          Returns stdout from the "git commit" shell command.
+
+            stdout from the "git commit" shell command.
+
         """
 
         # Lets escape all single quotes from the message
@@ -348,9 +467,10 @@ class GitRepo(object):
         """ Run git add on filename
 
             Arguments:
-                filename -- name of one file to add,
+                filename (str): name of one file to add,
+
             Returns:
-                The stdout from "git add" shell command.
+                str. The stdout from "git add" shell command.
         """
 
         # Escape all single quotes in filename:
@@ -365,10 +485,11 @@ class PerfData(object):
     """ Data Structure for a nagios perfdata string with multiple perfdata metric
 
     Example string:
+
     >>> perf = PerfData("load1=10 load2=10 load3=20 'label with spaces'=5")
     >>> perf.metrics
     ['load1'=10;;;;, 'load2'=10;;;;, 'load3'=20;;;;, 'label with spaces'=5;;;;]
-    >>> for i in perf.metrics: print i.label, i.value
+    >>> for i in perf.metrics: print("%s %s" % (i.label, i.value))
     load1 10
     load2 10
     load3 20
@@ -376,7 +497,9 @@ class PerfData(object):
     """
 
     def __init__(self, perfdatastring=""):
-        """ >>> perf = PerfData("load1=10 load2=10 load3=20") """
+        """
+        >>> perf = PerfData("load1=10 load2=10 load3=20") 
+        """
         self.metrics = []
         self.invalid_metrics = []
         # Hack: For some weird reason livestatus sometimes delivers perfdata in
@@ -396,6 +519,7 @@ class PerfData(object):
         """ Returns True if the every metric in the string is valid
 
         Example usage:
+
         >>> PerfData("load1=10 load2=10 load3=20").is_valid()
         True
         >>> PerfData("10b").is_valid()
@@ -413,18 +537,44 @@ class PerfData(object):
         return True
 
     def add_perfdatametric(self, perfdatastring="", label="", value="", warn="", crit="", min="", max="", uom=""):
-        """ Add a new perfdatametric to existing list of metrics.
+        """ 
+        Add a new perfdatametric to existing list of metrics.
 
-         Example:
-         >>> s = PerfData()
-         >>> s.add_perfdatametric("a=1")
-         >>> s.add_perfdatametric(label="utilization",value="10",uom="%")
+        Arguments:
+
+            perfdatastring (str): Complete perfdata string
+
+            label (str): Label section of the perfdata string
+
+            value (str): Value section of the perfdata string
+
+            warn (str): WARNING threshold
+
+            crit (str): CRITICAL threshold
+
+            min (str): Minimal value of control
+
+            max (str): Maximal value of control
+
+            uom (str): Measure unit (octets, bits/s, volts, ...)
+
+        Example:
+
+        >>> s = PerfData()
+        >>> s.add_perfdatametric("a=1")
+        >>> s.add_perfdatametric(label="utilization",value="10",uom="%")
         """
         metric=PerfDataMetric(perfdatastring=perfdatastring, label=label,value=value,warn=warn,crit=crit,min=min,max=max,uom=uom)
         self.metrics.append(metric)
 
     def get_perfdatametric(self, metric_name):
         """ Get one specific perfdatametric
+
+        Arguments:
+            metric_name (str): Name of the metric to return
+
+        Example:
+
         >>> s = PerfData("cpu=90% memory=50% disk_usage=20%")
         >>> my_metric = s.get_perfdatametric('cpu')
         >>> my_metric.label, my_metric.value
@@ -446,14 +596,37 @@ class PerfData(object):
 
 class PerfDataMetric(object):
 
-    """ Data structure for one single Nagios Perfdata Metric """
-    label = ""
-    value = ""
-    warn = ""
-    crit = ""
-    min = ""
-    max = ""
-    uom = ""
+    """
+    Data structure for one single Nagios Perfdata Metric 
+    
+
+    Attributes:
+
+        perfdatastring (str): Complete perfdata string
+
+        label (str): Label section of the perfdata string
+
+        value (str): Value section of the perfdata string
+
+        warn (str): WARNING threshold
+
+        crit (str): CRITICAL threshold
+
+        min (str): Minimal value of control
+
+        max (str): Maximal value of control
+
+        uom (str): Measure unit (octets, bits/s, volts, ...)
+
+    """
+
+    label = "" #: str. Label section of the perfdata string
+    value = "" #: str. Value section of the perfdata string
+    warn = "" #: str. WARNING threshold
+    crit = "" #: str CRITICAL threshold
+    min = "" #: str. Minimal value of control
+    max = "" #: str. Maximal value of control
+    uom = "" #: str. Measure unit (octets, bits/s, volts, ...)
 
     def __repr__(self):
         return "'%s'=%s%s;%s;%s;%s;%s" % (
@@ -473,16 +646,16 @@ class PerfDataMetric(object):
         """
         >>> p = PerfData(perfdatastring="size=10M;20M;;;")
         >>> metric = p.get_perfdatametric('size')
-        >>> print metric.label
+        >>> print(metric.label)
         size
-        >>> print metric.value
+        >>> print(metric.value)
         10
-        >>> print metric.uom
+        >>> print(metric.uom)
         M
         >>> p = PerfDataMetric(perfdatastring="'with spaces'=10")
-        >>> print p.label
+        >>> print(p.label)
         with spaces
-        >>> print p.value
+        >>> print(p.value)
         10
         """
         self.label = label
@@ -537,31 +710,39 @@ class PerfDataMetric(object):
             self.max = tmp.pop(0)
 
     def get_status(self):
-        """ Return nagios-style exit code (int 0-3) by comparing
+        """ 
+        Return nagios-style exit code (int 0-3) by comparing
 
-          self.value with self.warn and self.crit
+        Example: 
 
-          >>> PerfDataMetric("label1=10;20;30").get_status()
-          0
-          >>> PerfDataMetric("label2=25;20;30").get_status()
-          1
-          >>> PerfDataMetric("label3=35;20;30").get_status()
-          2
+        self.value with self.warn and self.crit
 
-          # Invalid metrics always return unknown
-          >>> PerfDataMetric("label3=35;invalid_metric").get_status()
-          3
+        >>> PerfDataMetric("label1=10;20;30").get_status()
+        0
+        >>> PerfDataMetric("label2=25;20;30").get_status()
+        1
+        >>> PerfDataMetric("label3=35;20;30").get_status()
+        2
+
+        Invalid metrics always return unknown
+
+        >>> PerfDataMetric("label3=35;invalid_metric").get_status()
+        3
         """
+
         try:
             status = pynag.Plugins.check_threshold(self.value, warning=self.warn, critical=self.crit)
         except pynag.Utils.PynagError:
             status = 3
+
         return status
 
     def is_valid(self):
-        """ Returns True if all Performance data is valid. Otherwise False
+        """ 
+        Returns True if all Performance data is valid. Otherwise False
 
         Example Usage:
+
         >>> PerfDataMetric("load1=2").is_valid()
         True
         >>> PerfDataMetric("load1").is_valid()
@@ -579,6 +760,7 @@ class PerfDataMetric(object):
         >>> PerfDataMetric("'label with spaces=0'").is_valid()
         False
         """
+
         if self.label in (None, ''):
             return False
 
@@ -607,13 +789,19 @@ class PerfDataMetric(object):
         return True
 
     def reconsile_thresholds(self):
-        """ Convert threshold from new threshold syntax to current one, for backwards compatibility """
-        print 'yeah', self
+        """ 
+        Convert threshold from new threshold syntax to current one, for backwards compatibility 
+        """
+
         self.warn = reconsile_threshold(self.warn)
         self.crit = reconsile_threshold(self.crit)
 
     def split_value_and_uom(self, value):
-        """ get value="10M" and return (10,"M")
+        """ 
+        Example:
+
+        get value="10M" and return (10,"M")
+
         >>> p = PerfDataMetric()
         >>> p.split_value_and_uom( "10" )
         ('10', '')
@@ -641,30 +829,66 @@ class PerfDataMetric(object):
             return '', ''
         return tmp[0]
 
+    def get_dict(self):
+        """
+        Returns a dictionary which contains this class' attributes.
+
+        Returned dict example::
+
+            {
+                'label': self.label,
+                'value': self.value,
+                'uom': self.uom,
+                'warn': self.warn,
+                'crit': self.crit,
+                'min': self.min,
+                'max': self.max,
+            }
+        """
+
+        return {
+            'label': self.label,
+            'value': self.value,
+            'uom': self.uom,
+            'warn': self.warn,
+            'crit': self.crit,
+            'min': self.min,
+            'max': self.max,
+        }
 
 def grep(objects, **kwargs):
-    """  Returns all the elements from array that match the keywords in **kwargs
-
-    TODO: Refactor pynag.Model.ObjectDefinition.objects.filter() and reuse it here.
-    Arguments:
-        array -- a list of dict that is to be searched
-        kwargs -- Any search argument provided will be checked against every dict
-    Examples:
-    array = [
-    {'host_name': 'examplehost', 'state':0},
-    {'host_name': 'example2', 'state':1},
-    ]
-    grep_dict(array, state=0)
-    # should return [{'host_name': 'examplehost', 'state':0},]
 
     """
+    Returns all the elements from array that match the keywords in **kwargs
+
+    See documentation for pynag.Model.ObjectDefinition.objects.filter() for 
+    example how to use this.
+
+    Arguments:
+
+        objects (list of dict): list to be searched
+
+        kwargs (str): Any search argument provided will be checked against every dict
+
+    Examples::
+
+        array = [
+        {'host_name': 'examplehost', 'state':0},
+        {'host_name': 'example2', 'state':1},
+        ]
+        grep_dict(array, state=0)
+        # should return [{'host_name': 'examplehost', 'state':0},]
+
+    """
+
     # Input comes to us as a key/value dict.
     # We will flatten this out into a tuble, because if value
     # is a list, it means the calling function is doing multible search on
     # the same key
     search = []
     for k, v in kwargs.items():
-        if type(v) == type([]):
+        # We need the actual array in "v" for __in and __notin
+        if type(v) == type([]) and not (k.endswith('__in') or k.endswith('__notin')):
             for i in v:
                 search.append((k, i))
         else:
@@ -675,7 +899,7 @@ def grep(objects, **kwargs):
         v_str = str(v)
         if k.endswith('__contains'):
             k = k[:-len('__contains')]
-            expression = lambda x: v_str in str(x.get(k))
+            expression = lambda x: x.get(k) and v_str in str(x.get(k))
         elif k.endswith('__notcontains'):
             k = k[:-len('__notcontains')]
             expression = lambda x: not v_str in str(x.get(k))
@@ -702,11 +926,11 @@ def grep(objects, **kwargs):
             regex = re.compile(str(v))
             expression = lambda x: regex.search(str(x.get(k)))
         elif k.endswith('__in'):
-            k = k = k[:-len('__in')]
+            k = k[:-len('__in')]
             expression = lambda x: str(x.get(k)) in v
         elif k.endswith('__notin'):
-            k = k = k[:-len('__notin')]
-            expression = lambda x: str(x.get(k)) in v
+            k = k[:-len('__notin')]
+            expression = lambda x: str(x.get(k)) not in v
         elif k.endswith('__has_field'):
             k = k[:-len('__has_field')]
             expression = lambda x: v_str in AttributeList(x.get(k)).fields
@@ -724,15 +948,30 @@ def grep(objects, **kwargs):
 
 
 def grep_to_livestatus(*args, **kwargs):
-    """ Converts from pynag style grep syntax to livestatus filter syntax.
+
+    """ 
+    Converts from pynag style grep syntax to livestatus filter syntax.
 
     Example:
+
         >>> grep_to_livestatus(host_name='test')
         ['Filter: host_name = test']
-
         >>> grep_to_livestatus(service_description__contains='serv')
         ['Filter: service_description ~ serv']
+        >>> grep_to_livestatus(service_description__isnot='serv')
+        ['Filter: service_description != serv']
+        >>> grep_to_livestatus(service_description__contains=['serv','check'])
+        ['Filter: service_description ~ serv']
+        >>> grep_to_livestatus(service_description__contains='foo', contacts__has_field='admin')
+        ['Filter: contacts >= admin', 'Filter: service_description ~ foo']
+        >>> grep_to_livestatus(service_description__has_field='foo')
+        ['Filter: service_description >= foo']
+        >>> grep_to_livestatus(service_description__startswith='foo')
+        ['Filter: service_description ~ ^foo']
+        >>> grep_to_livestatus(service_description__endswith='foo')
+        ['Filter: service_description ~ foo$']
     """
+
     result = list(args)  # Args go unchanged back into results
     for k, v in kwargs.items():
         if isinstance(v, list) and len(v) > 0:
@@ -746,6 +985,24 @@ def grep_to_livestatus(*args, **kwargs):
         elif k.endswith('__isnot'):
             k = k[:-len('__isnot')]
             my_string = "Filter: %s != %s" % (k, v)
+        elif k.endswith('__startswith'):
+            k = k[:-len('__startswith')]
+            my_string = "Filter: %s ~ ^%s" % (k, v)
+        elif k.endswith('__endswith'):
+            k = k[:-len('__endswith')]
+            my_string = "Filter: %s ~ %s$" % (k, v)
+        elif k == 'WaitObject':
+            my_string = "WaitObject: %s" % (v,)
+        elif k == 'WaitCondition':
+            my_string = "WaitCondition: %s" % (v,)
+        elif k == 'WaitTrigger':
+            my_string = "WaitTrigger: %s" % (v,)
+        elif k == 'WaitTimeout':
+            my_string = "WaitTimeout: %s" % (v,)
+        elif k in ('Limit', 'limit'):
+            my_string = "Limit: %s" % (v,)
+        elif k in ('Filter', 'filter'):
+            my_string = "Filter: %s" % (v,)
         else:
             my_string = "Filter: %s = %s" % (k, v)
         result.append(my_string)
@@ -754,29 +1011,37 @@ def grep_to_livestatus(*args, **kwargs):
 
 class AttributeList(object):
 
-    """ Parse a list of nagios attributes (e. contact_groups) into a parsable format
+    """ 
+    Parse a list of nagios attributes (e. contact_groups) into a parsable 
+    format.
 
-    This makes it handy to mangle with nagios attribute values that are in a comma seperated format.
+    This makes it handy to mangle with nagios attribute values that are in a 
+    comma seperated format.
+
+    Typical comma-seperated format in nagios configuration files looks something
+    like this::
 
-    Typical comma-seperated format in nagios configuration files looks something like this:
         contact_groups     +group1,group2,group3
 
-    Example:
+    Example::
+
         >>> i = AttributeList('+group1,group2,group3')
-        >>> print "Operator is:", i.operator
-        Operator is: +
-        >>> print i.fields
+        >>> i.operator
+        '+'
+        >>> i.fields
         ['group1', 'group2', 'group3']
 
-        if your data is already in a list format you can use it directly:
+        # if your data is already in a list format you can use it directly:
         >>> i = AttributeList(['group1', 'group2', 'group3'])
-        >>> print i.fields
+        >>> i.fields
         ['group1', 'group2', 'group3']
 
-        white spaces will be stripped from all fields
+        # white spaces will be stripped from all fields
         >>> i = AttributeList('+group1, group2')
-        >>> print i
+        >>> i
         +group1,group2
+        >>> i.fields
+        ['group1', 'group2']
 
     """
 
@@ -785,7 +1050,7 @@ class AttributeList(object):
         self.fields = []
 
         # this is easy to do if attribue_name is unset
-        if not value or value == '':
+        if not value or value == 'null':
             return
 
         # value in this case should usually be a comma seperated string, but sometimes
@@ -818,68 +1083,96 @@ class AttributeList(object):
         return self.__str__()
 
     def insert(self, index, object):
-        """ Same as list.insert()
+        """  Same as list.insert()
 
-        >>> i = AttributeList('group1,group2,group3')
-        >>> i.insert(1, 'group4')
-        >>> print i.fields
-        ['group1', 'group4', 'group2', 'group3']
+        Args:
+            object: Any object that will be inserted into self.fields (usually a string)
 
+        Example:
+            >>> i = AttributeList('group1,group2,group3')
+            >>> i.insert(1, 'group4')
+            >>> i.fields
+            ['group1', 'group4', 'group2', 'group3']
         """
+
         return self.fields.insert(index, object)
 
     def append(self, object):
-        """ Same as list.append()
+        """ Same as list.append():
 
-        >>> i = AttributeList('group1,group2,group3')
-        >>> i.append('group5')
-        >>> print i.fields
-        ['group1', 'group2', 'group3', 'group5']
+        Args:
+            object: Item to append into self.fields (typically a string)
 
+        Example:
+            >>> i = AttributeList('group1,group2,group3')
+            >>> i.append('group5')
+            >>> i.fields
+            ['group1', 'group2', 'group3', 'group5']
         """
         return self.fields.append(object)
 
     def count(self, value):
-        """ Same as list.count()
+        """  Same as list.count()
 
-        >>> i = AttributeList('group1,group2,group3')
-        >>> i.count('group3')
-        1
+        Args:
+            value: Any object that might exist in self.fields (string)
+
+        Returns:
+            The number of occurances that 'value' has in self.fields
+
+        Example:
+            >>> i = AttributeList('group1,group2,group3')
+            >>> i.count('group3')
+            1
         """
+
         return self.fields.count(value)
 
     def extend(self, iterable):
         """ Same as list.extend()
 
-        >>> i = AttributeList('group1,group2,group3')
-        >>> i.extend(['group4', 'group5'])
-        >>> print i.fields
-        ['group1', 'group2', 'group3', 'group4', 'group5']
+        Args:
+            iterable:   Any iterable that list.extend() supports
+
+        Example:
+            >>> i = AttributeList('group1,group2,group3')
+            >>> i.extend(['group4', 'group5'])
+            >>> i.fields
+            ['group1', 'group2', 'group3', 'group4', 'group5']
         """
         return self.fields.extend(iterable)
 
     def index(self, value, start=0, stop=None):
         """ Same as list.index()
 
-        >>> i = AttributeList('group1,group2,group3')
-        >>> i.index('group2')
-        1
-        >>> i.index('group3', 2, 5)
-        2
+        Args:
+            value:  object to look for in self.fields
+            start:  start at this index point
+            stop:   stop at this index point
 
+        Returns:
+            The index of 'value' (integer)
+
+        Examples:
+            >>> i = AttributeList('group1,group2,group3')
+            >>> i.index('group2')
+            1
+            >>> i.index('group3', 2, 5)
+            2
         """
+
         if stop is None:
             stop = len(self.fields)
         return self.fields.index(value, start, stop)
 
     def reverse(self):
-        """ Same as list.reverse()
-
-        >>> i = AttributeList('group1,group2,group3')
-        >>> i.reverse()
-        >>> print i.fields
-        ['group3', 'group2', 'group1']
+        """  Same as list.reverse()
 
+        Examples:
+            >>> i = AttributeList('group1,group2,group3')
+            >>> i.reverse()
+            >>> i.fields
+            ['group3', 'group2', 'group1']
         """
 
         return self.fields.reverse()
@@ -887,35 +1180,41 @@ class AttributeList(object):
     def sort(self):
         """ Same as list.sort()
 
-        >>> i = AttributeList('group3,group1,group2')
-        >>> i.sort()
-        >>> print i.fields
-        ['group1', 'group2', 'group3']
-
-
+        Examples:
+            >>> i = AttributeList('group3,group1,group2')
+            >>> i.sort()
+            >>> print(i.fields)
+            ['group1', 'group2', 'group3']
         """
+
         return self.fields.sort()
 
     def remove(self, value):
         """ Same as list.remove()
 
-        >>> i = AttributeList('group1,group2,group3')
-        >>> i.remove('group3')
-        >>> print i.fields
-        ['group1', 'group2']
+        Args:
+            value:  The object that is to be removed
 
+        Examples:
+            >>> i = AttributeList('group1,group2,group3')
+            >>> i.remove('group3')
+            >>> i.fields
+            ['group1', 'group2']
         """
+
         return self.fields.remove(value)
 
     def __iter__(self):
-        """ Same as list.__iter__()
+        """ 
+        Same as list.__iter__()
 
         >>> mylist = AttributeList('group1,group2,group3')
-        >>> for i in mylist: print i
+        >>> for i in mylist: print(i)
         group1
         group2
         group3
         """
+
         return self.fields.__iter__()
 
 
@@ -924,11 +1223,23 @@ class PluginOutput:
     """ This class parses a typical stdout from a nagios plugin
 
     It splits the output into the following fields:
+
     * Summary
     * Long Output
     * Perfdata
 
-    Example UsagE:
+    Attributes:
+
+        summary (str): Summary returned by the plugin check
+
+        long_output (str)
+
+        perfdata (str): Data returned by the plugin as a string
+
+        parsed_perfdata: perfdata parsed and split
+
+    Example Usage:
+
     >>> p = PluginOutput("Everything is ok | load1=15 load2=10")
     >>> p.summary
     'Everything is ok '
@@ -940,10 +1251,10 @@ class PluginOutput:
     ['load1'=15;;;;, 'load2'=10;;;;]
 
     """
-    summary = None
+    summary = None #: str. Summary returned by the plugin check
     long_output = None
-    perfdata = None
-    parsed_perfdata = None
+    perfdata = None #: str. Data returned by the plugin as a string
+    parsed_perfdata = None #: Perfdata parsed and split
 
     def __init__(self, stdout):
         if not stdout:
@@ -968,18 +1279,19 @@ class PluginOutput:
         self.perfdata = perfdata.strip()
         self.parsed_perfdata = PerfData(perfdatastring=perfdata)
 
-
 class defaultdict(dict):
 
-    """ This is an alternative implementation of collections.defaultdict.
+    """ 
+    This is an alternative implementation of collections.defaultdict.
 
     Used as a fallback if using python 2.4 or older.
 
-    Usage:
-    try:
-        from collections import defaultdict
-    except ImportError:
-        from pynag.Utils import defaultdict
+    Usage::
+
+        try:
+            from collections import defaultdict
+        except ImportError:
+            from pynag.Utils import defaultdict
     """
 
     def __init__(self, default_factory=None, *a, **kw):
@@ -1024,13 +1336,18 @@ class defaultdict(dict):
 
 
 def reconsile_threshold(threshold_range):
-    """ Take threshold string as and normalize it to the format supported by plugin development team
+    """ 
+    Take threshold string as and normalize it to the format supported by plugin 
+    development team
 
-    the input (usually a string in the form of 'the new threshold syntax') is a string in the form of x..y
+    The input (usually a string in the form of 'the new threshold syntax') is a 
+    string in the form of x..y
 
-    the output will be a compatible string in the older nagios plugin format @x:y
+    The output will be a compatible string in the older nagios plugin format 
+    @x:y
+
+    Examples:
 
-    examples
     >>> reconsile_threshold("0..5")
     '@0:5'
     >>> reconsile_threshold("inf..5")
@@ -1046,6 +1363,7 @@ def reconsile_threshold(threshold_range):
     >>> reconsile_threshold("10..inf")
     '~:10'
     """
+
     threshold_range = str(threshold_range)
     if not '..' in threshold_range:
         return threshold_range
@@ -1076,12 +1394,14 @@ def reconsile_threshold(threshold_range):
 
 
 def synchronized(lock):
-    """ Synchronization decorator
+    """ 
+    Synchronization decorator
 
     Use this to make a multi-threaded method synchronized and thread-safe.
 
-    Use the decorator like so:
-    @pynag.Utils.synchronized(pynag.Utils.rlock)
+    Use the decorator like so::
+
+        @pynag.Utils.synchronized(pynag.Utils.rlock)
     """
     def wrap(f):
         def newFunction(*args, **kw):
@@ -1090,5 +1410,73 @@ def synchronized(lock):
                 return f(*args, **kw)
             finally:
                 lock.release()
+        newFunction.__name__ = f.__name__
+        newFunction.__module__ = f.__module__
         return newFunction
     return wrap
+
+
+def cache_only(func):
+    def wrap(*args, **kwargs):
+        print "aggressive caching on"
+        pynag.Model.ObjectFetcher._cache_only = True
+        try:
+            return func(*args, **kwargs)
+        finally:
+            pynag.Model.ObjectFetcher._cache_only = False
+            print "Aggressive caching off"
+    wrap.__name__ = func.__name__
+    wrap.__module__ = func.__module__
+    return wrap
+
+
+def send_nsca(code, message, nscahost, hostname=None, service=None, nscabin="send_nsca", nscaconf=None):
+    """ 
+    Send data via send_nsca for passive service checks
+
+    Arguments:
+
+        code (int): Return code of plugin.
+
+        message (str): Message to pass back.
+
+        nscahost (str): Hostname or IP address of NSCA server.
+        
+        hostname (str): Hostname the check results apply to.
+        
+        service (str): Service the check results apply to.
+        
+        nscabin (str): Location of send_nsca binary. If none specified whatever
+                       is in the path will be used.
+        
+        nscaconf (str): Location of the NSCA configuration to use if any.
+
+    Returns:
+
+        [result,stdout,stderr] of the command being run
+    """
+
+    if not hostname:
+        hostname = node()
+
+    # Build command
+    command = [nscabin, '-H', nscahost]
+    if nscaconf:
+        command += ['-c', nscaconf]
+
+    # Just in case, status code was sent in as an integer:
+    code = str(code)
+
+    # Build the input string
+    if service:
+        input_string = '\t'.join([hostname, service, code, message]) + '\n'
+    else:
+        input_string = '\t'.join([hostname, code, message]) + '\n'
+
+    # Execute command
+
+    proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
+    stdout, stderr = proc.communicate(input=input_string)
+    result = proc.returncode, stdout, stderr
+
+    return result
